%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Q-NET: 1D Tests
% 
% Calculate Integrals
%    I  over [-1,1]  and 
%    Isub over [.5,1] 
% of 4 test functions 
%
% Step 1: Train proxy
% Step 2: Calculate I
% Step 3: Calculate Isub
% Step 4: Output
% Step 5: Plot
% 
% Proxy object p usage: 
%    p = Proxy(x,y,k,...)
%    p.Integral(-1,1) 
% 
%
% Same for d>1
% Only change is the training input 
%   x is d x N
%   y is 1 x N
%%%%%%%%%%%%%%%%%%%%%%%%%%
fv=[] ;
fv{1} = @(x) (x<.5).*x.^2 + (x>.5).*x ;                                             % test function 1
fv{2} = @(x) (x<.5).*sin(2*pi*x) + (x>.175).*(x<.625)*.5  ;                         % test function 2
fv{3} = @(x) (x<.5).*sin(20*pi*x) + (x>.175).*(x<.625)*.5 + (x>.7).*cos(5*pi*x) ;   % test function 3
fv{4} = @(x) sin(8*pi*x) >0 ;                                                       % test function 4

% settings
k = 100 ;               % number of neurons 
N = 180 ;               % number of training sample locations
nbatches = 1 ;         % set >1 if training data will not fit in memory
nreps = 1;             % Repeat training procedure with iterative re-initialization
nepochs = 3;           % Need more epochs if the function is difficult to learn
useGPU = 'no' ;        % 'yes' is Only beneficial if N and k are large

figure(1) 
clf
disp('-------------') ;    
for i=1:length(fv)
    testfunc = i ;
    
    f = fv{testfunc} ;
    x = linspace(-1,1,N) + rand(1,N)*1/(N-1);  % stratified samples of domain
    y = f(x) ;              % evaluate the function    
    
    % 1: Train proxy
    trainst = tic ;
    p = Proxy(x,y,k,nbatches,nreps,nepochs,useGPU) ; % proxy object p

    traintime = toc(trainst) ;

    
    % 2: Calculate integral
    intst = tic ;
    [p,I] = p.Integral(-1,1) ;
    inttime = toc(intst) ;

    % 3: Calculate integral over subdomain
    intsbst = tic ;
    Isub = p.IntegralSub(-1, 1, 0.5, 1) ;
    intsbtime = toc(intsbst) ;

    
    % 4: Output     
    disp('       ') ;
    disp(['Function ' num2str(i)]) ;
    disp(['    Training time: ', num2str(traintime) 's']) ;
    disp(['    Integration time: ', num2str(inttime) 's']) ;
    disp(['    Integral over [-1,1]: ', num2str(I) 's']) ;
    disp(['    Integral over [0.5,1]: ', num2str(Isub) 's']) ;
    disp('     ') ;
        
    % 5: Plot
    subplot(length(fv),1,i) ;
    dispres = 100 ;     % display resolution (# test points)
    xtest = linspace(-1,1,dispres) ;
    ftest = f(xtest) ;
    fwtest = p.EvalProxy(xtest) ;
    plot(x, y, '.g') ;
    hold on
    plot(xtest, fwtest, '-r') ;
    plot(xtest, ftest, ':k') ;
    
    hold off;
    pbaspect([4,1,1]) ;
    if i==1
        legend(gca, {'x_i', 'f_w', 'f'})
    end
    xlabel('x') 
    ylabel(['f_' num2str(i) '(x)']) 
end